var runAsWidget = false;
var targetDirectory = null;
var directoryWritable = false;
var pluginAvailable = false;
var JMA_QUAKE_HOME = "http://www.jma.go.jp/jp/quake/";
var runningCommand    = null;
var canvasParameter = null;
var PointImages = null; // cached images of observation points
var waitTimer = null;
var waitTimerCount = 0;
// scroller
var tableScroller = null;
// switcher
var panelSwitcher = null;
// transit information box
var boxMotion = null;

var EarthquakeInformation = new function() {
    this.url         = null;
    this.urlPrevious = null;
    this.urlNext     = null;

    this.intensity    = 0;
    this.depth        = 0;
    this.latitude     = 0.0;
    this.longitude    = 0.0;
    this.incidentTime = "";
    this.reportTime   = "";
    this.magnitude    = 0.0;

    this.gravityX = 0;

    this.observationPoints = new Array();
    this.intensityThreshold = 1;
    this.cacheSize = 0;

    if ( window.widget ) {
        this.intensityThreshold = parseInt( widget.preferenceForKey( "intensityThreshold" ) );
        this.cacheSize = parseInt( widget.preferenceForKey( "cacheSize" ) );
    }

    this.forwardAvailable  = false;
//     this.backwardAvailable = false;
//     this.latestAvailable   = false;

    this.toString = function() {
        var output = "";
        output += "URL \t" + this.url;
        output += "\nPREV\t" + this.urlPrevious;
        output += "\nNEXT\t" + this.urlNext;
        output += "\nITSY\t" + this.intensity;
        output += "\nDPTH\t" + this.depth;
        output += "\nPOS \t" + "( " + this.latitude + ", " + this.longitude + ")";
        output += "\nMGNT\t" + this.magnitude;
        return output;
    }

};


// class of observation points
function ObservationPoint( code, intensity ) {
    this.code = code;//"00000000";
    this.intensity = intensity;
    try {
        var ap = areaPosition[ code.substring( 0, 5 ) ];
        this.x = ap[ 0 ];
        this.y = ap[ 1 ];
        this.drawable = true;
         if ( canvasParameter.availability ) {
             this.x = parseFloat( this.x ) / 400 * canvasParameter.width;
             this.y = parseFloat( this.y ) / 190 * canvasParameter.height;
         }
         // offset by image size
         this.x -= 8;
         this.y -= 8;
    } catch ( e ) {
        this.x = -100;
        this.y = -100;
        this.drawable = false;
    }
}

// information box motion manager
function BoxMotion( target ) {
    var bs = document.defaultView.getComputedStyle( target );
    this.target = target;
    this.timer = null;
    var bx = parseInt( bs.getPropertyValue( "left" ) );
    var by = parseInt( bs.getPropertyValue( "top" ) );
    this.x = this.startX = this.endX = bx;
    this.y = this.startY = this.endY = by;
    this.count = 0;
    this.steps = 8;
    this.wait = 33;
}


function tr( key ) {
    return key;
//     if ( localizedStrings ) {
//         try {
//             return localizableStrings[ key ];
//         } catch ( e ) {
//             return key;
//         }
//     }
//     return key;
}


// Initializer
//
function init() {

    // check widget or browser
    if ( window.widget ) {
        runAsWidget = true;
    } else {
        runAsWidget = false;
    }

    // check plugin
    if ( window.EarthquakeFormatter ) {
        pluginAvailable = true;
        targetDirectory = EarthquakeFormatter.getUserHomeDirectory() + "/Library/Application\ Support/QuakeRecord";
        debug( targetDirectory );
        if ( EarthquakeFormatter.createDirectoryAtPath( targetDirectory ) ) {
            directoryWritable = true;
        } else {
            debug( "cannot create directory at " + targetDirectory );
        }
    } else {
        targetDirectory = ".";
    }

    // set button pushable
    var buttonlabels = [ "previousButton", "latestButton", "switchButton" ];
    for ( var i = 0; i < buttonlabels.length; i++ ) {
        _addButtonColor( buttonlabels[ i ] );
    }
    _resetButtonColor( "nextButton" );

    // canvas
    var canvas = document.getElementById( "quakeMap" );
    canvasParameter = { availability:false, width:0, height:0, canvas:null };
    if ( canvas ) {
        var cxt = canvas.getContext( "2d" );
        if ( cxt ) {
            canvasParameter.availability = true;
            var ps = document.defaultView.getComputedStyle( canvas );
            var width  = parseInt( ps.getPropertyValue( "width" ) );
            var height = parseInt( ps.getPropertyValue( "height" ) );
            canvasParameter.canvas = canvas;
            canvasParameter.width  = width;
            canvasParameter.height = height;
        } else {
            debug( "no canvas context" );
        }
        //debug( canvasParameter );
    }

    // scroller
    tableScroller = new Scroller( document.getElementById( 'tableFrame' ), document.getElementById( 'tableContainer' ), document.getElementById( 'scrollbar' ), document.getElementById( 'scrollthumb' ) );
    window.addEventListener( "mousewheel", listenMouseWheel, true );

    // panel switcher
    //debug( PanelSwitcher );
    panelSwitcher = new PanelSwitcher( document.getElementById( "mapPanel" ), document.getElementById( "slider" ), document.getElementById( "mapShadow" ) );

    // done button
    createGenericButton( document.getElementById( 'done' ), tr( "Done" ), flipToFront );
    createInformationButton( document.getElementById( "mapPanel" ), -25, -25, flipToBehind, false );

    // set preferences by default value
    var elem_intensity = document.getElementById( "item-intensity" );
    _setDefaultItem( elem_intensity, EarthquakeInformation.intensityThreshold );
    var elem_cache = document.getElementById( "item-cacheSize" );
    _setDefaultItem( elem_cache, EarthquakeInformation.cacheSize );


    // caching images
    PointImages = new Array( 7 );
    for ( var i = 0; i < 7; i++ ) {
        var img = new Image();
        img.src = "Images/quake" + i + ".png";
        PointImages[ i ] = img;
    }
    for ( var i = 0; i < 12; i++ ) {
        (new Image()).src = "Images/Wait" + i + ".png";
    }

    // remove cache files
    if ( EarthquakeInformation.cacheSize > 0 && runAsWidget ) {
        var command = "/usr/bin/python Scripts/removecache.py \"" + targetDirectory + "\" " + EarthquakeInformation.cacheSize;
        debug( command );
        widget.system( command, function() { debug( "cache removed" ); } );
    }

    // information box
    boxMotion = new BoxMotion( document.getElementById( "information" ) );

    setTimeout( loadHTML, 0 );
}

var logging = true;//false;
//debugging
function debug( message ) {
    if ( !logging ) return;
	if ( window.EarthquakeFormatter ) {
		EarthquakeFormatter.logMessage( message.toString() );
		return;
	}
	var elem = document.getElementById( "debug" );
	if ( elem == null ) {
		elem = document.createElement( "div" );
		elem.setAttribute( "id", "debug" );
		elem.style.position = "absolute";
		elem.style.left = "0px";
		elem.style.bottom = "0px";
		elem.style.right = "0px";
		elem.style.height = "130px";
		elem.style.color = "black";
		elem.style.background = "gray";
		elem.style.font = "monospace 10px";
		elem.style.zIndex = "1000";
		
		document.getElementById( "front" ).appendChild( elem );
	}
	elem.appendChild( document.createTextNode( message ) );
	elem.appendChild( document.createElement( "br" ) );
}

// replace regional information position
function transit() {
    if ( panelSwitcher ) {
        panelSwitcher.startTransition( _rearrange );
    }
}

function _rearrange() {
    if ( tableScroller ) {
        tableScroller.refresh();
    }
}

// start information box transition
function _startInformationTransition( x, y ) {
   if ( boxMotion.target == null ) return;
    if ( boxMotion.timer != null ) {
        clearInterval( boxMotion.timer );
        boxMotion.timer = null;
    }
    if ( boxMotion.endX == x
         && boxMotion.endY == y ) {
        return;
    }
    boxMotion.startX = boxMotion.x;
    boxMotion.startY = boxMotion.y;
    boxMotion.endX = x;//boxMotion.x;
    boxMotion.endY = y;//boxMotion.y;
    boxMotion.count = 0;
    boxMotion.timer = setInterval( _tickInformationTransition, boxMotion.wait );
    _tickInformationTransition();
}

// managere information box motion
function _tickInformationTransition() {
    if ( boxMotion.count >= boxMotion.steps ) {
        boxMotion.x = boxMotion.endX;
        boxMotion.y = boxMotion.endY;
        boxMotion.target.style.left = boxMotion.endX + "px";
        boxMotion.target.style.top  = boxMotion.endY + "px";
        clearInterval( boxMotion.timer );
        boxMotion.timer = null;
        return;
    }
    boxMotion.count ++;
    var rad = parseFloat( boxMotion.count ) / boxMotion.steps;
    var ratio = Math.sin( rad * Math.PI / 2 );
    boxMotion.x = ( boxMotion.endX - boxMotion.startX ) * ratio + boxMotion.startX;
    boxMotion.y = ( boxMotion.endY - boxMotion.startY ) * ratio + boxMotion.startY;
    boxMotion.target.style.left = boxMotion.x + "px";
    boxMotion.target.style.top  = boxMotion.y + "px";
}

// wait timer handlers
function startWaitTimer() {
    if ( waitTimer != null ) {
        stopWaitTimer();
    }
    waitTimer = setInterval( progressWaitTimer, 66 );
    progressWaitTimer();
}

function stopWaitTimer() {
    var elem_timer = document.getElementById( "wait" );
    elem_timer.style.display = "none";
    if ( waitTimer != null ) {
        clearInterval( waitTimer );
    }
    waitTimer = null;
//    waitTimerCount = 0;
}

function progressWaitTimer() {
    //debug( "progress " + waitTimerCount );
    var elem_timer = document.getElementById( "wait" );
    elem_timer.style.display = "block";
    elem_timer.style.backgroundImage = "url(Images/wait" + waitTimerCount + ".png)";
    waitTimerCount ++;
    if ( waitTimerCount >= 12 ) {
        waitTimerCount = 0;
    }
}

// load earthquake information from website
function loadHTML( url ) {
    if ( runAsWidget == false ) return;
    if ( runningCommand ) {
        runningCommand.cancel();
        runningCommand = null;
    }

    startWaitTimer();
    //disableButtons();

    var pycommand = "/usr/bin/python Scripts/retrievequake.py ";
    if ( _isValidURL( url ) == false ) {//url == null || url == undefined ) {
        url = JMA_QUAKE_HOME;
    }

    //EarthquakeInformation.url = url;
    pycommand += url;

    if ( directoryWritable ) {
        pycommand += " -d \"" + targetDirectory + "\"";
    }

    debug( pycommand );
    runningCommand = widget.system( pycommand, commandHandler );

}

function moveToNext() {
    if ( EarthquakeInformation.urlNext ) {
        loadHTML( EarthquakeInformation.urlNext );
    }
}

function moveToPrevious() {
    if ( EarthquakeInformation.urlPrevious ) {
        debug( EarthquakeInformation.urlPrevious );
        loadHTML( EarthquakeInformation.urlPrevious );
    }
}

function moveToLatest() {
    //if ( EarthquakeInformation.url ) {
    loadHTML( JMA_QUAKE_HOME );
        //}
}

function disableButtons() {
    // disable backward button
    // disable forward button
//     EarthquakeInformation.forwardAvailable  = false;
//     EarthquakeInformation.backwardAvailable = false;
//     EarthquakeInformation.latestAvailable   = false;
    debug( EarthquakeInformation );
}

function enableButtons() {
    debug( EarthquakeInformation );
//     EarthquakeInformation.forwardAvailable  = false;
//     EarthquakeInformation.backwardAvailable = false;
//     EarthquakeInformation.latestAvailable   = true;
//     // change button image
//     if ( EarthquakeInformation.urlPrevious ) {
//         EarthquakeInformation.backwardAvailable = true;
//         // change button image
//     }
//     if ( EarthquakeInformation.urlNext ) {
//         EarthquakeInformation.forwardAvailable  = true;
//         // change button image
//     }
}

function cancelLoading() {
    if ( runningCommand ) {
        runningCommand.cancel();
    }
    enableButtons();
}

// !commandHandler
//
function commandHandler( command ) {
    if ( !pluginAvailable ) return;
    stopWaitTimer()
    var error = command.errorString;
    if ( error != null && error.length > 0 ) {
        debug( error );
        var elem = document.getElementById( "information" );
        if ( elem ) {
            while ( elem.lastChild ) elem.removeChild( elem.lastChild );
            var message = "データの読み込みに失敗しました";
            elem.appendChild( document.createTextNode( tr( message ) ) );
        }        
        return;
    }
    var output = command.outputString;
    runningCommand = null;
    //debug( "OUTPUT" );
    if ( output ) {
        debug( output );
        var lines = output.split( "\n" );
        var filename = lines[ 0 ];

        // pass information to formatter
//         if ( EarthquakeFormatter.isReading() ) { 
//             var localCommand = command;
//             setTimeout( 100, new function() { 
//                             commandHandler( localCommand );
//                         } );
//             return;
//         }

        var url_current = EarthquakeInformation.url;
        EarthquakeFormatter.readEarthQuakeFile( filename );
        EarthquakeInformation.observationPoints = new Array();
        //debug( "current URL " + url_current );

        // information
        var direction = 0;
        var information = EarthquakeFormatter.getInformation();
        //debug( information );
        var ilines = information.split( "\n" );
        for ( var i = 0; i < ilines.length; i++ ) {
            var pos = ilines[ i ].search( ":" );
            if ( pos < 0 ) continue;
            var key   = ilines[ i ].substring( 0, pos );
            var value = ilines[ i ].substring( pos + 1, ilines[ i ].length );//- pos - 1 );
            //debug( key + "," + value );
            if ( key == "Report" ) {
                EarthquakeInformation.reportTime = _formatDate( value );
            } else if ( key == "Incident" ) {
                EarthquakeInformation.incidentTime = _formatDate( value );
            } else if ( key == "URL" ) {
                if ( _isValidURL( value ) ) {
                    EarthquakeInformation.url = value;
                } else {
                    EarthquakeInformation.url = null;
                }
                debug( "change to " + value );
            } else if ( key == "PrevURL" ) {
                if ( _isValidURL( value ) ) {
                    if ( url_current == value ) {
                        direction = 1;
//                     } else {
//                         debug( "PREV:not same " + url_current + ", " + value );
                    }
                    EarthquakeInformation.urlPrevious = value;
                } else {
                    EarthquakeInformation.urlPrevious = null;
                }
            } else if ( key == "NextURL" ) {
                if ( _isValidURL( value ) ) {
                    if ( url_current == value ) {
                        direction = -1;
//                     } else {
//                         debug( "NEXT:not same " + url_current + ", " + value );
                    }
                    EarthquakeInformation.urlNext = value;
                } else {
                    EarthquakeInformation.urlNext = null;
                }
            } else if ( key == "Magnitude" ) {
                EarthquakeInformation.magnitude = parseFloat( value );
            } else if ( key == "Latitude" ) {
                EarthquakeInformation.latitude = parseFloat( value );
            } else if ( key == "Longitude" ) {
                EarthquakeInformation.longitude = parseFloat( value );
            } else if ( key == "Max" ) {
                EarthquakeInformation.intensity = parseInt( value );
            } else if ( key == "Depth" ) {
                EarthquakeInformation.depth = value + "km";
            } else {
                var items = ilines[ i ].split( ":" );
                var point = new ObservationPoint( items[ 0 ], parseInt( items[ 1 ] ) );
                EarthquakeInformation.observationPoints.push( point );
            }
        }
        //debug( EarthquakeInformation );
        if ( EarthquakeInformation.intensity < EarthquakeInformation.intensityThreshold ) {
            if ( direction <= 0 ) {
                debug( "small earthquake, previous" );
                moveToPrevious();
            } else if ( direction > 0 ) {
                debug( "small earthquake, next" );
                moveToNext();
            } else {
                debug( "no direction detected" );
            }
            return;
        }

        EarthquakeInformation.observationPoints.sort( _intensityComparator );
//        repaint();
//         debug( ">> drawpoints 510" );
//         drawPoints();
//         showInformation();

        // table
        var tableContainer = document.getElementById( "tableContainer" );
        if ( tableContainer ) {
            var thtml = EarthquakeFormatter.getTableHTML();
            tableContainer.innerHTML = thtml;

//             if ( tableScroller ) {
//                 tableScroller.refresh();
//             }
//            debug( thtml );
            setTimeout( repaint, 100 );
        } else {
            repaint();
            debug( "no container" );
        }

    } else {
        debug( "no output" );
    }
}

// repaint graphics
// redraw points not to destruct canvas image
function repaint() {
//    debug( ">> drawpoints 537" );
    drawPoints();
    showInformation();
    _resetButtonColor( "nextButton" );
    if ( EarthquakeInformation.urlNext ) {
        _addButtonColor( "nextButton" );
    }
    _resetButtonColor( "previousButton" );
    if ( EarthquakeInformation.urlPrevious ) {
        _addButtonColor( "previousButton" );
    }

    //_setForwardAvailable( EarthquakeInformation.forwardAvailable );

    if ( tableScroller ) {
        tableScroller.refresh();
    }
}

// !drawPoints
function drawPoints() {
    if ( canvasParameter == null || canvasParameter.availability == false ) {
        debug( "canvas not available" );
        return;
    }

    var canvas = document.getElementById( "quakeMap" );
    var cxt = canvas.getContext( "2d" );

    var sum_x = 0;
    var num_drawn = 0;
    cxt.save();
    cxt.clearRect( 0, 0, canvasParameter.width, canvasParameter.height );

    var points = EarthquakeInformation.observationPoints;
    for ( var i = 0; i < points.length; i++ ) {
        var point = points[ i ];
        if ( point.drawable == false ) continue;
        var x = point.x;
        var y = point.y;
        var intensity = parseInt( point.intensity );
        sum_x += x;
        num_drawn ++;
        debug( "(" + x + ", " + y + ") : " + intensity );
        if ( intensity < 0 ) continue;
        if ( intensity > 6 ) intensity = 6;

        var img = PointImages[ intensity ];
        cxt.drawImage( img, x, y, 16, 16 );
    }
    if ( num_drawn > 0 ) {
        EarthquakeInformation.gravityX = parseFloat( sum_x ) / num_drawn;
    }

    // draw earthquake center
    var lat = EarthquakeInformation.latitude;
    var lon = EarthquakeInformation.longitude;

    // rotate 37.7
    {
        var sin = Math.sin( - 37.7 * Math.PI / 180.0 );
        var cos = Math.cos( - 37.7 * Math.PI / 180.0 );
        var s1  = 262.0 * cos -  87.0 * sin;
        var s2  = 262.0 * sin +  87.0 * cos;
        var t1  = 134.0 * cos - 119.0 * sin;
        var t2  = 134.0 * sin + 119.0 * cos;
        var u1 = t1 + ( lon - 135 ) / 5 * ( s1 - t1 );
        var u2 = t2 + ( lat - 35 ) / 5 * ( s2 - t2 );//* v2;
        var cx = u1 * cos + u2 * sin;
        var cy = - u1 * sin + u2 * cos;
        cx *= canvasParameter.width / 400;
        cy *= canvasParameter.height / 190;

        cxt.strokeStyle = "red";
        cxt.lineWidth = 1.0;
        cxt.moveTo( cx - 4, cy - 4 );
        cxt.lineTo( cx + 4, cy + 4 );
        cxt.stroke();
        cxt.moveTo( cx + 4, cy - 4 );
        cxt.lineTo( cx - 4, cy + 4 );
        cxt.stroke();

        // transfer information panel
        var elem = document.getElementById( "information" );
        if ( elem ) {
            if ( cx > canvasParameter.width / 2 ) {
                _startInformationTransition( 40, boxMotion.y );
            } else {
                _startInformationTransition( 208, boxMotion.y );
            }

        }
    }

    cxt.restore();
}


function showInformation() {
    var table = document.createElement( "table" );
    table.setAttribute( "id", "informationTable" );
    _addRow( table, tr( "時刻" ), EarthquakeInformation.incidentTime );
    _addRow( table, tr( "マグニチュード" ), EarthquakeInformation.magnitude );
    _addRow( table, tr( "最大震度" ), EarthquakeInformation.intensity );
    _addRow( table, tr( "震源地" ), "(" + EarthquakeInformation.latitude + "/" + EarthquakeInformation.longitude + ")" );
    _addRow( table, tr( "深さ" ), EarthquakeInformation.depth );
    _addRow( table, tr( "発表" ), EarthquakeInformation.reportTime );
    var elem = document.getElementById( "information" );
    if ( elem ) {
        while ( elem.lastChild ) elem.removeChild( elem.lastChild );
        elem.appendChild( table );
    }
}

// comparator by intensity
function _intensityComparator( p1, p2 ) {
    if ( p1.intensity < p2.intensity ) {
        return -1;
    } else if ( p1.intensity > p2.intensity ) {
        return 1;
    }
    return 0;
}

// add button listeners to respond to click
function _addButtonColor( id ) {
    var elem = document.getElementById( id );
    if ( elem ) {
        elem.addEventListener( 'mousedown', _darkenButton, true );
        elem.addEventListener( 'mouseup', _lightButton, true );
        elem.addEventListener( 'mouseout', _lightButton, true );
        elem.style.color = "black";
        elem.available = true;
    }
}
function _resetButtonColor( id ) {
    var elem = document.getElementById( id );
    if ( elem ) {
        elem.removeEventListener( 'mousedown', _darkenButton, true );
        elem.removeEventListener( 'mouseup', _lightButton, true );
        elem.removeEventListener( 'mouseout', _lightButton, true );
        elem.style.color = "gray";
        elem.available = false;
    }
}

function _darkenButton( event ) {
    var elem = event.srcElement;
    if ( elem.nodeType == 3 ) { // text
        elem = elem.parentNode;
        if ( elem.nodeType != 1 ) return;
    }
    elem.style.color = "gray";
}

function _lightButton( event ) {
    var elem = event.srcElement;
    if ( elem.nodeType == 3 ) { // text
        elem = elem.parentNode;
        if ( elem.nodeType != 1 ) return;
    }
    elem.style.color = "black";
}

// set the default value to preference option
function _setDefaultItem( element, defaultvalue ) {
    if ( element ) {
        var children = element.childNodes;
        for ( var i = 0; i < children.length; i++ ) {
            var child = children.item( i );
            debug( child + " " + child.nodeType );
            if ( child.nodeType == 1 && child.value == defaultvalue ) {
                child.setAttribute( "selected" );
            } else if ( child.nodeType == 1 ) {
                child.removeAttribute( "selected" );
            }
        }
    }
}

function _isValidURL( url ) {
    if ( url == undefined || url == null ) return false;
    if ( url.search( JMA_QUAKE_HOME ) == 0 ) {
        return true;
    }
    return false;
}

function _addRow( table, item1, item2 ) {
    var row = document.createElement( "tr" );
    var d1  = document.createElement( "td" );
    d1.appendChild( document.createTextNode( item1.toString() ) );
    row.appendChild( d1 );
    var d2  = document.createElement( "td" );
    d2.appendChild( document.createTextNode( item2.toString() ) );
    row.appendChild( d2 );
    table.appendChild( row );
}

function _formatDate( date ) {
    var items = date.split( "/" );
    if ( items.length >= 5 ) {
        return items[ 1 ] + "/" + items[ 2 ] + " " + items[ 3 ] + ":" + items[ 4 ];
    }
    return date;
}

function flipToBehind() {
    if ( panelSwitcher && panelSwitcher.isRunning() ) return;
    var front = document.getElementById( "front" );
    var panel = document.getElementById( "mapPanel" );
    var pref  = document.getElementById( "behind" );
    if ( runAsWidget ) {
        widget.prepareForTransition( "ToBack" );
        behind.style.display = "block";
        front.style.display = "none";
        pref.style.top = panel.style.top;
        setTimeout( "widget.performTransition();", 0 );
    } else {
        behind.style.display = "block";
        front.style.display = "none";
    }

}

function flipToFront() {
    var elem = document.getElementById( "item-intensity" );
    if ( elem && elem.value ) {
        EarthquakeInformation.intensityThreshold = parseInt( elem.value );
    }
    var elem_cache = document.getElementById( "item-cacheSize" );
    if ( elem_cache && elem_cache.value ) {
        EarthquakeInformation.cacheSize = parseInt( elem_cache.value );
    }
    if ( runAsWidget ) {
        widget.setPreferenceForKey( EarthquakeInformation.intensityThreshold, "intensityThreshold" );
        widget.setPreferenceForKey( EarthquakeInformation.cacheSize, "cacheSize" );
        debug( EarthquakeInformation.intensityThreshold + ", " + EarthquakeInformation.cacheSize );
    }

    var front = document.getElementById( "front" );
    var perf  = document.getElementById( "behind" );
    if ( runAsWidget ) {
        widget.prepareForTransition( "ToFront" );
        front.style.display = "block";
        behind.style.display = "none";
        setTimeout( "widget.performTransition();", 0 );
    } else {
        behind.style.display = "block";
        front.style.display = "none";
    }

}


function listenMouseWheel( event ) {
    var offset = event.wheelDelta;
    if ( tableScroller != null ) {
        if ( offset > 0 ) {
            tableScroller.tickUp();
        } else {
            tableScroller.tickDown();
        }
    }
}
